package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.BizTenant;
import cn.ezhizao.project.business.service.IBizTenantService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.domain.BizCompany;
import cn.ezhizao.project.business.service.IBizCompanyService;

/**
 * 客户名称Controller
 *
 * @author ezhizao
 * @date 2025-04-09
 */
@Api("公司管理")
@RestController
@RequestMapping("/business/company")
public class BizCompanyController extends BaseController
{
    @Resource
    private IBizCompanyService bizCompanyService;
    @Resource
    private IBizTenantService bizTenantService;

    /**
     * 查询客户名称列表
     */
//        @ApiOperation("获取公司详细")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "companyCode", value = "公司编码", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "companyName", value = "公司名称",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "companyAlias", value = "公司别名", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "tenantId", value = "租户ID",  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "mnemonicCode", value = "助记码",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "salemanCode", value = "业务员编码", dataType = "String", paramType = "path", dataTypeClass = String.class),
//    })
    @GetMapping("/list")
    public TableDataInfo list(BizCompany bizCompany) throws NoSuchFieldException, IllegalAccessException
    {
        List<BizCompany> list = bizCompanyService.getList(bizCompany);
        return getDataTable(list);
    }


    /**
     * 获取客户名称详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizCompanyService.getById(id));
    }

    /**
     * 新增客户名称
     */
//        @ApiOperation("新增公司")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "tenantId", value = "租户ID", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "companyCode", value = "公司编码", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "companyName", value = "公司名称", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "companyAlias", value = "公司别名", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "mnemonicCode", value = "助记码",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "salemanCode", value = "业务员编码", dataType = "String", paramType = "path", dataTypeClass = String.class),
//    })
    @Log(title = "客户名称", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizCompany bizCompany) throws NoSuchFieldException, IllegalAccessException
    {
        //tenant_id,companyCode,company_alias不能为空
        if(bizCompany.getTenantId() == null || bizCompany.getCompanyCode() == null || bizCompany.getCompanyCode() == "" || bizCompany.getCompanyAlias() == null || bizCompany.getCompanyAlias() ==""|| bizCompany.getCompanyName() == null || bizCompany.getCompanyName() ==""){
            return error("新增客户失败,必填项:厂别,公司编码,公司名称,公司别名不能为空");
        }
        //判断厂别是否存在
        List<BizTenant> tenantList = bizTenantService.query().eq("id", bizCompany.getTenantId()).list();
        if(tenantList.isEmpty()){
            return error("新增客户失败,厂别不存在");
        }
        //判断相同厂别不能有重复的公司编码
        List<BizCompany> companyList = bizCompanyService.query().eq("tenant_id", bizCompany.getTenantId()).eq("company_code", bizCompany.getCompanyCode()).list();
        if(!companyList.isEmpty()){
            return error("新增客户失败,相同厂别不能有重复的公司编码");
        }
        return toAjax(bizCompanyService.save(bizCompany));
    }

    /**
     * 修改客户名称
     */
//        @ApiOperation("修改公司")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "公司ID", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "tenantId", value = "厂别ID",  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "companyCode", value = "公司编码",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "companyName", value = "公司名称",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "companyName", value = "公司别名",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "mnemonicCode", value = "助记码",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "salemanCode", value = "业务员编码", dataType = "String", paramType = "path", dataTypeClass = String.class),
//    })
    @Log(title = "客户名称", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizCompany bizCompany) throws NoSuchFieldException, IllegalAccessException
    {
        if(bizCompany.getId() == null){
            return error("修改客户失败,id不能为空");
        }
        if(bizCompany.getCompanyCode() == "" ||bizCompany.getCompanyAlias() ==""|| bizCompany.getCompanyName() ==""){
            return error("修改客户失败,公司编码,公司名称,公司别名不能为空");
        }
        //判断厂别是否存在
        if(bizCompany.getTenantId() != null) {
            List<BizTenant> tenantList = bizTenantService.query().eq("id", bizCompany.getTenantId()).list();
            if (tenantList.isEmpty()) {
                return error("修改客户失败,厂别不存在");
            }
           if(bizCompany.getCompanyCode() !=null){
               List<BizCompany> companyList = bizCompanyService.query().eq("tenant_id", bizCompany.getTenantId()).eq("company_code", bizCompany.getCompanyCode()).ne("id", bizCompany.getId()).list();
               if(!companyList.isEmpty()){
                   return error("修改客户失败,相同厂别不能有重复的公司编码");
               }
           }
        }
        BizCompany oldCompany = bizCompanyService.getById(bizCompany.getId());
        //判断相同厂别不能有重复的公司编码
        if(bizCompany.getCompanyCode() !=null){
            List<BizCompany> companyList = bizCompanyService.query().eq("tenant_id", oldCompany.getTenantId()).eq("company_code", bizCompany.getCompanyCode()).ne("id", bizCompany.getId()).list();
            if(!companyList.isEmpty()){
                return error("修改客户失败,相同厂别不能有重复的公司编码");
            }
        }
        return toAjax(bizCompanyService.updateById(bizCompany));
    }

    /**
     * 删除客户名称
     */
//        @ApiOperation("删除公司")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "公司ID", required = true, dataType = "List<Long>", paramType = "path", dataTypeClass = List.class)
//    })
    @Log(title = "客户名称", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizCompanyService.removeBatchByIds(ids));
    }
}
