package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.constant.UserConstants;
import cn.ezhizao.common.utils.StringUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.mapper.BizTurnoverMapper;
import cn.ezhizao.project.business.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 部门Controller
 *
 * @author ezhizao
 * @date 2025-04-08
 */
//@Api("部门管理")
@RestController
@RequestMapping("/business/dept")
public class SysDeptController extends BaseController
{
    @Resource
    private ISysDeptService sysDeptService;
    @Resource
    private IBizTurnoverService bizTurnoverService;
    @Resource
    private IBizSupplierService bizSupplierService;
    @Resource
    private ISysDeptUserService sysDeptUserService;
    @Resource
    private IBizTenantService bizTenantService;
    @Resource
    HttpServletRequest request;

    /**
     * 查询部门列表
     */
//    @ApiOperation("获取部门详细")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "deptName", value = "部门名称", required = false, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "deptCode", value = "部门编码", required = false, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "deptId", value = "部门ID", required = false, dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "tenantId", value = "厂别ID", required = false, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
//    })
    @GetMapping("/list")
    public TableDataInfo list(SysDept sysDept) throws NoSuchFieldException, IllegalAccessException
    {
        List<SysDept> list = sysDeptService.getList(sysDept);
        return getDataTable(list);
    }
    /**
     * 新增部门
     */
//    @ApiOperation("新增部门")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "deptName", value = "部门名称", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "deptCode", value = "部门编码", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "tenantId", value = "厂别ID", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "isOutsourcing", value = "是否外协",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "supplierId", value = "外协商ID",  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "isWorkSection", value = "是否为工段",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "equimentFlag", value = "是否一机多批",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "isSortPackaging", value = "是否分选",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "parentId", value = "父类ID",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//    })
    @Log(title = "部门", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SysDept sysDept) throws NoSuchFieldException, IllegalAccessException
    {
        //判断必填项
        if(sysDept.getTenantId() ==null ||sysDept.getDeptName() ==null || sysDept.getDeptName() ==""||sysDept.getDeptCode() ==null || sysDept.getDeptCode() =="") {
           return error("新增部门失败，请求必填项：厂别、部门名称、部门编码不能为空");
        }
        if((sysDept.getIsOutsourcing() !=null && sysDept.getIsOutsourcing() ==1) && sysDept.getSupplierId() ==null) {
            return error("新增部门失败，外协商不能为空");
        }
        if(sysDept.getIsOutsourcing() ==null && sysDept.getSupplierId() !=null) {
            return error("新增部门失败，是否外协应为是");
        }
        //是否判断，只能是null，0,1
        if(sysDept.getIsOutsourcing() !=null && !(sysDept.getIsOutsourcing() ==0 || sysDept.getIsOutsourcing() ==1)){
            return error("新增部门失败，是否外协只能是0或1");
        }
        if(sysDept.getIsWorkSection() !=null &&sysDept.getIsWorkSection() !="" && !(sysDept.getIsWorkSection().equals("0") || sysDept.getIsWorkSection().equals("1"))){
            return error("新增部门失败，是否为工段只能是0或1");
        }
        if(sysDept.getEquimentFlag() !=null && !(sysDept.getEquimentFlag() ==0 || sysDept.getEquimentFlag() ==1)){
            return error("新增部门失败，是否一机多批只能是0或1");
        }
        if(sysDept.getIsSortPackaging() !=null && !(sysDept.getIsSortPackaging() ==0 || sysDept.getIsSortPackaging() ==1)){
            return error("新增部门失败，是否分选只能是0或1");
        }
        //厂别判断
        List<BizTenant> tenantList = bizTenantService.query().eq("id", sysDept.getTenantId()).list();
        if(tenantList.isEmpty()){
            return error("新增部门失败，厂别不存在");
        }
        //外协商判断
        if((sysDept.getIsOutsourcing() !=null && sysDept.getIsOutsourcing() ==1) && sysDept.getSupplierId() !=null) {
            SysDept dept = new SysDept();
            dept.setDeptId(0L);
            dept.setIsInnerOutsource(1);
            List<BizSupplier> supplierList = bizSupplierService.getListByDept(dept);
            if(supplierList.isEmpty()){
                return error("新增部门失败，外协商都被使用，请新增外协商");
            }else{
                if(supplierList.stream().filter(supplier -> supplier.getId().equals(sysDept.getSupplierId())).findFirst().orElse(null) == null){
                    return error("新增部门失败，外协商已被使用，请重新选择");
                }
            }
        }
        if(sysDept.getParentId() == null){
            sysDept.setParentId(0L);
        }
            if (UserConstants.NOT_UNIQUE.equals(sysDeptService.checkDeptNameUnique(sysDept)))
            {
                return error("新增部门'" + sysDept.getDeptName() + "'失败，部门名称已存在");
            }
            if (UserConstants.NOT_UNIQUE.equals(sysDeptService.checkDeptCodeUnique(sysDept)))
            {
                return error("新增部门'" + sysDept.getDeptName() + "'失败，部门编码已存在");
            }
            sysDept.setCreateBy(getUsername());
            int i = sysDeptService.insertDept(sysDept);
            //  新增默认摆放区域信息
            if (i > 0) {
                BizTurnover turnover = new BizTurnover();
                turnover.setStatus(9);
                turnover.setCode("默认区");
                turnover.setTenantId(sysDept.getTenantId());
                turnover.setDeptId(sysDept.getDeptId());
                bizTurnoverService.save(turnover);
            }
            return success();
    }

    /**
     * 修改部门
     */
//    @ApiOperation("修改部门")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "deptId", value = "部门ID", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "deptName", value = "部门名称",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "deptCode", value = "部门编码",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "tenantId", value = "厂别ID",  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "isOutsourcing", value = "是否外协",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "supplierId", value = "外协商ID",  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "isWorkSection", value = "是否为工段",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "equimentFlag", value = "是否一机多批",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "isSortPackaging", value = "是否分选",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "parentId", value = "父类ID",  dataType = "Integer", paramType = "path", dataTypeClass = Integer.class),
//    })
    @Log(title = "部门", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SysDept sysDept) throws NoSuchFieldException, IllegalAccessException
    {
        //判断必填项
        if(sysDept.getDeptId() ==null) {
            return error("修改部门失败，部门id不能为空");
        }
        if((sysDept.getIsOutsourcing() !=null && sysDept.getIsOutsourcing() ==1) && sysDept.getSupplierId() ==null) {
            return error("修改部门失败，外协商不能为空");
        }
        if(sysDept.getIsOutsourcing() ==null && sysDept.getSupplierId() !=null) {
            return error("修改部门失败，是否外协应为是");
        }
        //是否判断，只能是null，0,1
        if(sysDept.getIsOutsourcing() !=null && !(sysDept.getIsOutsourcing() ==0 || sysDept.getIsOutsourcing() ==1)){
            return error("修改部门失败，是否外协只能是0或1");
        }
        if(sysDept.getIsWorkSection() !=null && sysDept.getIsWorkSection() !="" && !(sysDept.getIsWorkSection().equals("0") || sysDept.getIsWorkSection().equals("1"))){
            return error("修改部门失败，是否为工段只能是0或1");
        }
        if(sysDept.getEquimentFlag() !=null && !(sysDept.getEquimentFlag() ==0 || sysDept.getEquimentFlag() ==1)){
            return error("修改部门失败，是否一机多批只能是0或1");
        }
        if(sysDept.getIsSortPackaging() !=null && !(sysDept.getIsSortPackaging() ==0 || sysDept.getIsSortPackaging() ==1)){
            return error("修改部门失败，是否分选只能是0或1");
        }
        //厂别判断
        List<BizTenant> tenantList = bizTenantService.query().eq("id", sysDept.getTenantId()).list();
        if(tenantList.isEmpty()){
            return error("修改部门失败，厂别不存在");
        }
        //外协商判断
        if((sysDept.getIsOutsourcing() !=null && sysDept.getIsOutsourcing() ==1) && sysDept.getSupplierId() !=null) {
            SysDept dept = new SysDept();
            dept.setDeptId(0L);
            dept.setIsInnerOutsource(1);
            List<BizSupplier> supplierList = bizSupplierService.getListByDept(dept);
            if(supplierList.isEmpty()){
                return error("修改部门失败，外协商都被使用，请新增外协商");
            }else{
                if(supplierList.stream().filter(supplier -> supplier.getId().equals(sysDept.getSupplierId())).findFirst().orElse(null) == null){
                    return error("修改部门失败，外协商已被使用，请重新选择");
                }
            }
        }
        if(sysDept.getParentId() == null){
            sysDept.setParentId(0L);
        }
        if (UserConstants.NOT_UNIQUE.equals(sysDeptService.checkDeptNameUnique(sysDept))) {
            return error("修改部门'" + sysDept.getDeptName() + "'失败，部门名称已存在");
        }else if (UserConstants.NOT_UNIQUE.equals(sysDeptService.checkDeptCodeUnique(sysDept))) {
            return error("修改部门'" + sysDept.getDeptName() + "'失败，部门编码已存在");
        }
        else if (sysDept.getParentId().equals(sysDept.getDeptId())) {
            return error("修改部门'" + sysDept.getDeptName() + "'失败，上级部门不能是自己");
        } else if (StringUtils.equals(UserConstants.DEPT_DISABLE, sysDept.getStatus()) && sysDeptService.selectNormalChildrenDeptById(sysDept.getDeptId()) > 0) {
            return error("该部门包含未停用的子部门！");
        }
        sysDept.setUpdateBy(getUsername());
        sysDeptService.updateDept(sysDept);
        return success();
    }

    /**
     * 删除部门
     */
//    @ApiOperation("删除部门")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "deptId", value = "部门ID", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
//    })
    @Log(title = "部门", businessType = BusinessType.DELETE)
	@DeleteMapping("/{deptId}")
    public AjaxResult remove(@PathVariable Long deptId)
    {
        SysDept dept = sysDeptService.getDeptByDeptId(deptId);
        if (dept == null){
            return error("部门不存在");
        }
        List<SysDeptUser> deptUserList = sysDeptUserService.query().in("dept_id", deptId).eq("deleted",0).list();
        if (deptUserList.size() > 0)
        {
            return error("部门存在用户,不允许删除");
        }
        if (sysDeptService.hasChildByDeptId(deptId))
        {
            return warn("存在下级部门,不允许删除");
        }
        return toAjax(sysDeptService.deleteDeptById(deptId));
    }
}
