package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.constant.UserConstants;
import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.StringUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.BizTenant;
import cn.ezhizao.project.business.service.IBizTenantService;
import cn.ezhizao.project.system.domain.SysUser;
import cn.ezhizao.project.system.service.ISysUserService;
import com.baomidou.mybatisplus.annotation.TableField;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 用户信息Controller
 *
 * @author ezhizao
 * @date 2025-04-09
 */
@Api("员工管理")
@RestController
@RequestMapping("/business/user")
public class SysUserController extends BaseController
{
    @Resource
    private ISysUserService sysUserService;
    @Resource
    private IBizTenantService bizTenantService;

    /**
     * 查询用户信息列表
     */
//            @ApiOperation("获取员工详细")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "tenantId", value = "厂别ID", dataType = "Long", paramType = "path", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "userName", value = "用户编码",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "nickName", value = "用户姓名", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "status", value = "账号状态",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//    })
    @GetMapping("/list")
    public TableDataInfo list(SysUser sysUser) throws NoSuchFieldException, IllegalAccessException
    {
//        if(sysUser.getTenantId() == null){
//            TableDataInfo error = new TableDataInfo();
//            error.setCode(500);
//            error.setMsg("查询失败，厂别不能为空");
//            return error;
//        }
        List<SysUser> list = sysUserService.selectEmployeeList(sysUser);
        return getDataTable(list);
    }


    /**
     * 新增用户信息
     */
//            @ApiOperation("新增用户")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "userName", value = "用户编码", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "nickName", value = "用户姓名", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "usageCompanyId", value = "使用单位ID", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "contractCompanyId", value = "合同单位ID", required = true,  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "sex", value = "用户性别", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "status", value = "账号状态", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "email", value = "用户邮箱", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "phoneNumber", value = "用户电话", dataType = "String", paramType = "path", dataTypeClass = String.class),
//    })
    @Log(title = "用户信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SysUser sysUser) throws NoSuchFieldException, IllegalAccessException
    {
        if(sysUser.getUserName() == null || sysUser.getUserName() == ""||sysUser.getNickName() == null || sysUser.getNickName() == "" || sysUser.getUsageCompanyId() == null
                ||sysUser.getContractCompanyId() == null ||sysUser.getPassword()==null||sysUser.getPassword()=="" ){
            return error("新增用户失败，员工姓名，员工编码，合同单位，使用单位，登录密码必填项不能为空");
        }
        //判断员工状态，用户性别的值
        if(sysUser.getSex() !=null && sysUser.getSex() !="" && (!sysUser.getSex().equals("男") && !sysUser.getSex().equals("女"))){
            return error("新增用户失败，用户性别的值只能为男或女");
        }
        if(sysUser.getSex() == null || sysUser.getSex() == ""){
            sysUser.setSex("男");
        }
        if(sysUser.getStatus() !=null && sysUser.getStatus() != "" && (!sysUser.getStatus().equals("0") && !sysUser.getStatus().equals("1"))){
            return error("新增用户失败，员工状态的值只能为0或1");
        }

        //判断厂别是否存在
        BizTenant usageTenant = bizTenantService.getById(sysUser.getUsageCompanyId());
        if(usageTenant == null){
            return error("新增用户失败，使用单位不存在");
        }
        sysUser.setUsageCompanyName(usageTenant.getOrgName());
        sysUser.setTenantId(sysUser.getUsageCompanyId());
        BizTenant contractTenant = bizTenantService.getById(sysUser.getContractCompanyId());
        if(contractTenant == null){
            return error("新增用户失败，合同单位不存在");
        }
        sysUser.setUsageCompanyName(contractTenant.getOrgName());
        if (UserConstants.NOT_UNIQUE.equals(sysUserService.checkUserNameUnique(sysUser))) {
            return error("新增用户'" + sysUser.getUserName() + "'失败，登录账号已存在");
        } else if (StringUtils.isNotEmpty(sysUser.getPhoneNumber())
                && UserConstants.NOT_UNIQUE.equals(sysUserService.checkPhoneUnique(sysUser))) {
            return error("新增用户'" + sysUser.getUserName() + "'失败，手机号码已存在");
        } else if (StringUtils.isNotEmpty(sysUser.getEmail())
                && UserConstants.NOT_UNIQUE.equals(sysUserService.checkEmailUnique(sysUser))) {
            return error("新增用户'" + sysUser.getUserName() + "'失败，邮箱账号已存在");
        }
        sysUser.setCreateBy(getUsername());
        sysUser.setPassword(SecurityUtils.encryptPassword(sysUser.getPassword()));
        sysUser.setUserType("02");
        return success(sysUserService.insertUser(sysUser));
    }

    /**
     * 修改用户信息
     */
//    @ApiOperation("新增用户")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "userId", value = "用户ID", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "userName", value = "用户编码", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "nickName", value = "用户姓名",  dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "password", value = "密码", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "usageCompanyId", value = "使用单位ID",  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "contractCompanyId", value = "合同单位ID",  dataType = "Long", paramType = "path", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "sex", value = "用户性别", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "status", value = "账号状态", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "email", value = "用户邮箱", dataType = "String", paramType = "path", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "phoneNumber", value = "用户电话", dataType = "String", paramType = "path", dataTypeClass = String.class),
//    })
    @Log(title = "用户信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SysUser sysUser) throws NoSuchFieldException, IllegalAccessException
    {
        if(sysUser.getUserId() == null){
            return error("修改用户失败，用户ID不能为空");
        }
        if(sysUser.getUserName() == ""|| sysUser.getNickName() == "" ||sysUser.getPassword()=="" ){
            return error("修改用户失败，员工姓名，员工编码，登录密码不能为空");
        }
        sysUserService.checkUserAllowed(sysUser);
        sysUserService.checkUserDataScope(sysUser.getUserId());
        //判断员工状态，用户性别的值
        if(sysUser.getSex() !=null &&sysUser.getSex() !="" && (!sysUser.getSex().equals("男") && !sysUser.getSex().equals("女"))){
            return error("修改用户失败，用户性别的值只能为男或女");
        }
        if(sysUser.getStatus() !=null &&sysUser.getStatus() !="" && (!sysUser.getStatus().equals("0") && !sysUser.getStatus().equals("1"))){
            return error("修改用户失败，员工状态的值只能为0或1");
        }
        if(sysUser.getUsageCompanyId() != null){
            //判断厂别是否存在
            BizTenant usageTenant = bizTenantService.getById(sysUser.getUsageCompanyId());
            if(usageTenant == null){
                return error("修改用户失败，使用单位不存在");
            }
            sysUser.setUsageCompanyName(usageTenant.getOrgName());
            sysUser.setTenantId(sysUser.getUsageCompanyId());
        }
        if(sysUser.getContractCompanyId() != null){
            BizTenant contractTenant = bizTenantService.getById(sysUser.getContractCompanyId());
            if(contractTenant == null){
                return error("修改用户失败，合同单位不存在");
            }
            sysUser.setContractCompanyName(contractTenant.getOrgName());
        }
        if (UserConstants.NOT_UNIQUE.equals(sysUserService.checkUserNameUnique(sysUser))) {
            return error("修改用户'" + sysUser.getUserName() + "'失败，登录账号已存在");
        } else if (StringUtils.isNotEmpty(sysUser.getPhoneNumber())
                && UserConstants.NOT_UNIQUE.equals(sysUserService.checkPhoneUnique(sysUser))) {
            return error("修改用户'" + sysUser.getUserName() + "'失败，手机号码已存在");
        } else if (StringUtils.isNotEmpty(sysUser.getEmail())
                && UserConstants.NOT_UNIQUE.equals(sysUserService.checkEmailUnique(sysUser))) {
            return error("修改用户'" + sysUser.getUserName() + "'失败，邮箱账号已存在");
        }
        sysUser.setUpdateBy(getUsername());
        return toAjax(sysUserService.updateUser(sysUser));
    }

    /**
     * 删除用户信息
     */
//            @ApiOperation("删除用户")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "userId", value = "用户ID", required = true, dataType = "List<Long>", paramType = "path", dataTypeClass = List.class)
//    })
    @Log(title = "用户信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{userIds}")
    public AjaxResult remove(@PathVariable List<Long> userIds)
    {
        List<SysUser> sysUsers = sysUserService.selectUserListByIds(userIds);
        if(userIds.size() != sysUsers.size()){
            return error("删除用户失败，存在用户ID不存在");
        }
        return toAjax(sysUserService.deleteUserByIds(userIds));
    }
}
