package cn.ezhizao.project.tool.swagger.util;

import cn.ezhizao.common.utils.StringUtils;
import cn.ezhizao.project.tool.swagger.dto.Request;
import cn.ezhizao.project.tool.swagger.dto.Response;
import cn.ezhizao.project.tool.swagger.dto.Table;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import org.springframework.stereotype.Component;

import java.util.*;
@Component
public class SwaggerParser {
    private static String projectPath = "http://192.168.31.126:8083/business/dept/list";
    public static List<Table> generateInterfaceDoc(JSONArray apiInfos){
        List<Table> list = new LinkedList();
        for (Object info : apiInfos) {
            JSONObject api = (JSONObject) info;
            //解析paths
//            LinkedHashMap<String, LinkedHashMap> paths = (LinkedHashMap) api.getJSONObject("paths");
            String group = api.getString("group");
            LinkedHashMap<String, Object> paths = JSON.parseObject(api.getJSONObject("paths").toJSONString(),LinkedHashMap.class, Feature.OrderedField);
            if (paths != null) {
                Iterator<Map.Entry<String, Object>> iterator = paths.entrySet().iterator();
                while (iterator.hasNext()) {
                    Table table = new Table();
                    List<Request> requestList = new LinkedList<Request>();
                    String requestType = "";

                    Map.Entry<String, Object> next = iterator.next();
                    //得到url
                    String url = next.getKey();
                    LinkedHashMap<String, Object> value = JSON.parseObject(next.getValue().toString(),LinkedHashMap.class, Feature.OrderedField);
                    //得到请求方式，输出结果类似为 get/post/delete/put 这样
                    Set<String> requestTypes = value.keySet();
                    for (String str : requestTypes) {
                        requestType += str + "/";
                    }
                    Iterator<Map.Entry<String, Object>> it2 = value.entrySet().iterator();
                    //解析请求,得到请求类型
                    Map.Entry<String, Object> get = it2.next();
                    JSONObject getValue = (JSONObject)get.getValue();
                    String title = (String) ((List) getValue.get("tags")).get(0);//得到大标题
                    String tag = String.valueOf(getValue.get("summary"));
                    //请求体
                    JSONArray parameters = getValue.getJSONArray("parameters");
//                    ArrayList parameters = (ArrayList) getValue.get("parameters");
                    if (parameters != null && parameters.size() > 0) {
                        for (int i = 0; i < parameters.size(); i++) {
                            Request request = new Request();
                            LinkedHashMap<String, Object> param = JSON.parseObject(parameters.get(i).toString(),LinkedHashMap.class, Feature.OrderedField);
                            request.setDescription(String.valueOf(param.get("description")));
                            request.setName(String.valueOf(param.get("name")));
                            request.setType(String.valueOf(param.get("type")));
                            request.setParamType(String.valueOf(param.get("in")));
                            request.setRequire((Boolean) param.get("required"));
                            requestList.add(request);
                        }
                    }
                    //返回体，比较固定
                    List<Response> responseList = listResponse();
                    //模拟一次HTTP请求,封装请求体和返回体，如果是Restful的文档可以再补充
                    if (requestType.contains("post")) {
                        Map<String, Object> stringStringMap = toPostBody(requestList);
                        table.setRequestParam(stringStringMap.toString());
                        String post =HttpUtil.post(projectPath+ url, stringStringMap);
                        table.setResponseParam(post);
                    } else if (requestType.contains("get")) {
                        String s = toGetHeader(requestList);
                        table.setResponseParam(s);
                        String getStr = HttpUtil.get(projectPath + url + s);
                        table.setResponseParam(getStr);
                    }

                    //封装Table
                    table.setGroup(group);
                    table.setTitle(title);
                    table.setUrl(url);
                    table.setTag(tag);
                    table.setResponseForm("application/json");
                    table.setRequestType(StringUtils.removeEnd(requestType, "/"));
                    table.setRequestList(requestList);
                    table.setResponseList(responseList);
                    list.add(table);
                }
            }
            return list;
        }
        return null;
    }

    //封装返回信息，可能需求不一样，可以自定义
    private static List<Response> listResponse() {
        List<Response> responseList = new LinkedList<Response>();
        responseList.add(new Response("受影响的行数", "counts", null));
        responseList.add(new Response("结果说明信息", "msg", null));
        responseList.add(new Response("是否成功", "success", null));
        responseList.add(new Response("返回对象", "data", null));
        responseList.add(new Response("错误代码", "errCode", null));
        return responseList;
    }

    //封装post请求体
    private static Map<String, Object> toPostBody(List<Request> list) {
        Map<String, Object> map = new HashMap<>(16);
        if (list != null && list.size() > 0) {
            for (Request request : list) {
                String name = request.getName();
                String type = request.getType();
                switch (type) {
                    case "string":
                        map.put(name, "string");
                        break;
                    case "integer":
                        map.put(name, "0");
                        break;
                    case "double":
                        map.put(name, "0.0");
                        break;
                    default:
                        map.put(name, "null");
                        break;
                }
            }
        }
        return map;
    }

    //封装get请求头
    private static String toGetHeader(List<Request> list) {
        StringBuffer stringBuffer = new StringBuffer();
        if (list != null && list.size() > 0) {
            for (Request request : list) {
                String name = request.getName();
                String type = request.getType();
                switch (type) {
                    case "string":
                        stringBuffer.append(name+"&=string");
                        break;
                    case "integer":
                        stringBuffer.append(name+"&=0");
                        break;
                    case "double":
                        stringBuffer.append(name+"&=0.0");
                        break;
                    default:
                        stringBuffer.append(name+"&=null");
                        break;
                }
            }
        }
        String s = stringBuffer.toString();
        if ("".equalsIgnoreCase(s)){
            return "";
        }
        return "?" + StringUtils.removeStart(s, "&");
    }
}