package cn.ezhizao.project.tool.swagger.util;

import cn.ezhizao.project.tool.swagger.dto.Request;
import cn.ezhizao.project.tool.swagger.dto.Response;
import cn.ezhizao.project.tool.swagger.dto.Table;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import org.apache.poi.xwpf.usermodel.*;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STMerge;
import org.springframework.stereotype.Component;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
@Component
public class WordGenerator {
    public void test(List<Table> tables) {
        // 创建文档
        XWPFDocument document = new XWPFDocument();

        // 添加标题
        XWPFParagraph titleParagraph = document.createParagraph();
        titleParagraph.setAlignment(ParagraphAlignment.CENTER);
        XWPFRun titleRun = titleParagraph.createRun();
        titleRun.setText("后台API接口文档");
        titleRun.setBold(true);
        titleRun.setFontSize(20);

        // 添加接口信息表格
        // 获取接口详细信息数据

//        for (int i = 0; i < interfaceList.size(); i++) {
//            Table table = interfaceList.get(i);
        createTable(document, tables);
//        }

        // 保存文档
        try (FileOutputStream fileOutputStream = new FileOutputStream("D:/数据导入/后台API接口文档.docx")) {
            document.write(fileOutputStream);
            System.out.println("接口文档生成成功！");

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 在文档中添加一个表格，将接口详细信息填充到表格中
     *
     * @param document 文档对象
     */
    private XWPFDocument createTable(XWPFDocument document, List<Table> tables) {
        tables.forEach(table -> {
            // 添加小标题
            XWPFParagraph tagParagraph = document.createParagraph();
            XWPFRun run = tagParagraph.createRun();
            run.setText(table.getGroup());
            run.setBold(true);
            run.setFontSize(16);

            // 添加表格
            XWPFTable xwpfTable = document.createTable(table.getRequestList().size() + 5, 6);
            xwpfTable.setWidth("100%");

            // 设置表头
            setCellText(xwpfTable.getRow(0).getCell(0), table.getTag(),true);
            setCellText(xwpfTable.getRow(1).getCell(0), "URL");
            setCellText(xwpfTable.getRow(2).getCell(0), "请求类型");
            setCellText(xwpfTable.getRow(3).getCell(0), "返回值类型");

            mergeCellsHorizontal(xwpfTable.getRow(0), 0, 5);
            setCellText(xwpfTable.getRow(4).getCell(0), "请求参数", true);
            setCellText(xwpfTable.getRow(4).getCell(1), "参数名", true);
            setCellText(xwpfTable.getRow(4).getCell(2), "数据类型", true);
//            setCellText(xwpfTable.getRow(4).getCell(3), "参数类型", true);
            setCellText(xwpfTable.getRow(4).getCell(3), "是否必填", true);
            setCellText(xwpfTable.getRow(4).getCell(4), "说明", true);

            // 填充表格内容
            mergeCellsHorizontal(xwpfTable.getRow(1), 1, 5);
            setCellText(xwpfTable.getRow(1).getCell(1), table.getUrl());

            mergeCellsHorizontal(xwpfTable.getRow(2), 1, 5);
            setCellText(xwpfTable.getRow(2).getCell(1), table.getRequestType());

            mergeCellsHorizontal(xwpfTable.getRow(3), 1, 5);
            setCellText(xwpfTable.getRow(3).getCell(1), table.getResponseForm());

            for (int i = 0; i < table.getRequestList().size(); i++) {
                Request request = table.getRequestList().get(i);
                if (ObjectUtils.isEmpty(request)) {
                    break;
                }
                XWPFTableRow row = xwpfTable.getRow(i + 5);
                setCellText(row.getCell(0), request.getDescription());
                setCellText(row.getCell(1), request.getName());
                setCellText(row.getCell(2), request.getType());
//                setCellText(row.getCell(3), request.getParamType());
                setCellText(row.getCell(3), request.getRequire() ? "是" : "否");
                setCellText(row.getCell(4), request.getRemark());
            }
            // 返回
            XWPFTableRow respRow = xwpfTable.createRow();

            mergeCellsHorizontal(respRow, 0, 1);
            mergeCellsHorizontal(respRow, 2, 3);
            mergeCellsHorizontal(respRow, 4, 5);

            setCellText(respRow.getCell(0), "返回参数", true);
            setCellText(respRow.getCell(2), "参数名", true);
            setCellText(respRow.getCell(4), "说明", true);

            for (int i = 0; i < table.getResponseList().size(); i++) {
                Response response = table.getResponseList().get(i);
                XWPFTableRow dataRow = xwpfTable.createRow();
                mergeCellsHorizontal(dataRow, 0, 1);
                mergeCellsHorizontal(dataRow, 2, 3);
                mergeCellsHorizontal(dataRow, 4, 5);
                setCellText(dataRow.getCell(0), response.getDescription());
                setCellText(dataRow.getCell(2), response.getName());
                setCellText(dataRow.getCell(4), response.getRemark());
            }
        });
        return document;
    }

    private void setCellText(XWPFTableCell cell, String text) {
        setCellTextCenterAlignment(cell);
        XWPFParagraph paragraph = cell.getParagraphs().get(0);
        XWPFRun run = paragraph.createRun();
        run.setText(text);
    }

    private void setCellText(XWPFTableCell cell, String text, boolean izBold) {
        setCellTextCenterAlignment(cell);
        XWPFParagraph paragraph = cell.getParagraphs().get(0);
        XWPFRun run = paragraph.createRun();
        run.setText(text);
//        run.setBold(izBold);
        cell.setColor("5881AF");
    }

    /**
     * 单元格文字居中
     *
     * @param cell
     */
    private void setCellTextCenterAlignment(XWPFTableCell cell) {
        List<XWPFParagraph> paragraphs = cell.getParagraphs();
        if (paragraphs.isEmpty()) {
            // 如果单元格中没有段落，则创建一个新的段落对象
            XWPFParagraph paragraph = cell.addParagraph();
            paragraph.setAlignment(ParagraphAlignment.CENTER);
        } else {
            // 否则，获取第一个段落对象并设置对齐方式
            XWPFParagraph firstParagraph = paragraphs.get(0);
            firstParagraph.setAlignment(ParagraphAlignment.CENTER);
        }
    }
    /**
     * 合并单元格
     *
     * @param row
     */
    public void mergeCellsHorizontal(XWPFTableRow row, int fromCell, int toCell) {
        for (int cellIndex = fromCell; cellIndex <= toCell; cellIndex++) {
            XWPFTableCell cell = row.getCell(cellIndex);
            if (cellIndex == fromCell) {
                // The first merged cell is set with RESTART merge value
                cell.getCTTc().addNewTcPr().addNewHMerge().setVal(STMerge.RESTART);
            } else {
                // Cells which join (merge) the first one, are set with CONTINUE
                cell.getCTTc().addNewTcPr().addNewHMerge().setVal(STMerge.CONTINUE);
            }
        }
    }
}